package fms

import (
	"hongze/fms_api/global"
	"time"
)

// ContractServiceDetail 合同服务内容详情/表单数据
type ContractServiceDetail struct {
	Id                 int       `gorm:"primaryKey;column:id" json:"id"`
	ContractServiceId  int       `gorm:"column:contract_service_id" json:"contract_service_id" description:"合同服务id"`
	ContractRegisterId int       `gorm:"column:contract_register_id" json:"contract_register_id" description:"合同id"`
	ServiceTemplateId  int       `gorm:"column:service_template_id" json:"service_template_id" description:"服务模板id"`
	Col1               string    `gorm:"column:col_1" json:"col_1" description:"第1列数据"`
	Col2               string    `gorm:"column:col_2" json:"col_2" description:"第2列数据"`
	Col3               string    `gorm:"column:col_3" json:"col_3" description:"第3列数据"`
	Col4               string    `gorm:"column:col_4" json:"col_4" description:"第4列数据"`
	Col5               string    `gorm:"column:col_5" json:"col_5" description:"第5列数据"`
	Col6               string    `gorm:"column:col_6" json:"col_6" description:"第6列数据"`
	Col7               string    `gorm:"column:col_7" json:"col_7" description:"第7列数据"`
	CreateTime         time.Time `gorm:"column:create_time" json:"create_time" description:"创建时间"`
}

func (cs *ContractServiceDetail) TableName() string {
	return "contract_service_detail"
}

// GetContractServiceDetailByTemplateId 根据服务模板ID获取对应的套餐表格数据详情
func GetContractServiceDetailByTemplateId(serviceTemplateId int) (items []*ContractServiceDetail, err error) {
	items = make([]*ContractServiceDetail, 0)
	err = global.DEFAULT_MYSQL.Model(ContractServiceDetail{}).
		Where("service_template_id = ? AND contract_service_id = 0", serviceTemplateId).
		Order("id ASC").
		Find(&items).Error
	return
}

type ContractServiceDetailAddReq struct {
	CanEdit    bool   `json:"CanEdit" description:"是否可编辑"`
	Type       string `json:"Type" description:"类型"`
	Value      string `json:"Value" description:"值"`
	ValueId    []int  `json:"ValueId" description:"类型"`
	HeadName   string `json:"HeadName" description:"表头名称"`
	Tag        string `json:"Tag" description:"表头名称"`
	RowDisable bool   `json:"RowDisable" description:"该行是否禁用操作"`
	RowName    string `json:"RowName" description:"该行关联名称"`
	Name       string `json:"Name" description:"行名称"`
}

// GetContractServiceDetailByServiceId 根据合同服务id获取对应的套餐表格数据详情
func GetContractServiceDetailByServiceId(serviceId int) (list []*ContractServiceDetail, err error) {
	err = global.DEFAULT_MYSQL.Model(ContractServiceDetail{}).
		Where("contract_service_id = ?", serviceId).
		Order("id ASC").
		Find(&list).Error
	return
}
